=begin
/****************************************************************************
** $Id: xform.rb,v 1.1.1.1 2000/09/19 07:56:09 horie Exp $
**
** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
**
** self file is part of an example program for Qt.  self example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/
=end
require "qt"
include Qt



class XFormControl < QFrame

def initialize parent, name
    @newMatrix = RSignal.new("Object") #( QWMatrix )
    @newText = RSignal.new("const char *") #( const char * )
    @newFont = RSignal.new("Object") #( const QFont & )
    @newMode = RSignal.new("Object") #( bool image )

    @rotLCD	= QLCDNumber.new( 4, self, "rotateLCD" )
    @rotS	= QSlider.new(0,99,10,0, QSlider::Horizontal, self,
				  "rotateSlider" )
    @shearLCD	= QLCDNumber.new( 5,self, "shearLCD" )
    @shearS	= QSlider.new( 0,99,10,0,QSlider::Horizontal, self,
				  "shearSlider" )
    @mirror	= QCheckBox.new( "",self, "mirrorCheckBox" )
    @textEd	= QLineEdit.new( self, "text" )
    @f		= QPushButton.new( "",self, "text" )
    @imgCheckBox = QCheckBox.new( "",self, "fontOrImage" )

    @rotLCD.setGeometry( 10, 10, 100, 60 )
    @rotLCD.displayText( "  0'" )

    @rotS.setGeometry( 10, 80, 100, 15 )
    @rotS.setRange( -180, 180 )
    @rotS.setValue( 0 )
    Qt::connect( @rotS, QSIGNAL("valueChanged(int)"), self,"newMtx" )

    @shearLCD.setGeometry( 10, 105, 100, 60 )
    @shearLCD.displayText( "0.00" )

    @shearS.setGeometry( 10, 175, 100, 15 )
    @shearS.setRange( -25, 25 )
    @shearS.setValue( 0 )
    Qt::connect( @shearS, QSIGNAL("valueChanged(int)"), self,"newMtx" )

    @mirror.setGeometry( 10, 200, 100, 15 )
    @mirror.setText( "Mirror" )
    Qt::connect( @mirror, QSIGNAL("clicked()"), self, "newMtx" )

    @imgCheckBox.setGeometry( 10, 220, 100, 15 )
    @imgCheckBox.setText( "Image" )
    Qt::connect( @imgCheckBox, QSIGNAL("clicked()"), self,"toggleMode" )

    @textEd.setGeometry( 10, 245, 100, 20 )
    @textEd.setText( "Troll" )
    Qt::connect( @textEd, QSIGNAL("textChanged(const char*)"),@newText )

    @f.setGeometry( 10, 275, 100, 20 )
    @f.setText( "Select font..." )
    Qt::connect( @f, QSIGNAL("clicked()"), self, "selectFont" )

    @magS    = nil
    @fs	    = nil
    catchEvent
end
def newMatrix
    @newMatrix
end
def newText
 @newText
end
def newFont
 @newFont
end
def newMode
  @newMode
end
#/*
#    Called whenever the user has changed one of the matrix parameters
#    (i.e. rotate, shear or magnification)
#*/
def newMtx dum=0

    m = QWMatrix.new
    if(! (@magS.nil?) )
    if ( @imgCheckBox.isChecked() )
	magVal = 1.0*@magS.value()/100
	m.scale( magVal, magVal )
    end
    end
    shearVal = 1.0*@shearS.value()/25
    m.shear( shearVal, shearVal )
    m.rotate( @rotS.value() )
    if ( @mirror.isChecked() )
	m.scale( 1, -1 )
	m.rotate( 180 )
    end

    tmp = "#{shearVal}"
    if ( shearVal >= 0 )
	tmp = " " + tmp
    end
    @shearLCD.displayText( tmp )

    rot = @rotS.value()
    if ( rot < 0 )
	rot = rot + 360
    end
    tmp = "#{rot}'"
    @rotLCD.displayText( tmp )
    @newMatrix.sendWith( m )
end


def selectFont()

    if (@fs.nil?)
	@fs   = FontSelect.new(nil,"")
	Qt::connect( @fs, QSIGNAL("destroyed()"), self,"selectFontDestroyed" )
	Qt::connect( @fs.newFont,self,"fontSelected" )
	@fs.setGeometryQRect( QRect.new( 100, 200, 380, 260 ) )
	@fsUp = false
    end
    @fsUp = !@fsUp
    if ( @fsUp )
	@fs.show()
    else
	@fs.hide()
    end
end

=begin
/*
    Called when the user has closed the SelectFont dialog via the
    window manager.
*/
=end

def selectFontDestroyed()

    @fs = 0
end

def fontSelected( font )

    @imgCheckBox.setChecked( FALSE )
    @newFont.sendWith( font )
    toggleMode()
end

=begin
/*
    Toggles between image and text mode.
*/
=end

def toggleMode()

    if ( @magS == nil )
	@magS = QSlider.new(0,99,10,0, QSlider::Horizontal, self,
			       "magnifySlider" )
	@magS.setGeometry( 10, 375, 100, 15 )
	@magS.setRange( 0, 400 )
	@magS.setValue( 100 )
	Qt::connect( @magS, QSIGNAL("valueChanged(int)"), self,"newMtx" )
	@magLCD = QLCDNumber.new( 4,self, "shearLCD" )
	@magLCD.setGeometry( 10, 305, 100, 60 )
	@magLCD.displayText( "100" )
	Qt::connect( @magS, QSIGNAL("valueChanged(int)"), @magLCD, QSLOT("display(int)"))
    end
    @newMode.sendWith( @imgCheckBox.isChecked() )
    newMtx()
    if (!@magS.nil?)
    if ( @imgCheckBox.isChecked() )
	@magS.show()
	@magLCD.show()
    else
	@magS.hide()
	@magLCD.hide()
    end
    end
    QApplication::flushX()
end
end

#/*
#  ShowXForm displays a text or a pixmap (QPixmap) using a coordinate
#  transformation matrix (QWMatrix)
#*/

class ShowXForm < QWidget

def initialize( parent, name )
    setFont( QFont.new( "Charter", 48, QFont::Bold ) )
    setBackgroundColor( QColor("white") )
    @usingPixmap = false
    @eraseRect = QRect.new( 0, 0, 0, 0 )
    @mtx = QWMatrix.new
    catchEvent
end

def pixmapMode
    @usingPixmap
end

def paintEvent( e )
    showIt()
end

def resizeEvent( e )
    @eraseRect = QRect.new( width()/2, height()/2, 0, 0 )
end

def setText( s )
    @text = s
    showIt()
end

def setMatrix( w )
    @mtx = w
    showIt()
end

def setFont( f )
    @usingPixmap = FALSE
    #QWidget::setFont( f )
    super
end

def setPixmap(  pm )
    @pix		= pm
    @usingPixmap = TRUE
    showIt()
end

def setPixmapMode( enable )
    @usingPixmap = enable
end

def showIt()

    p = QPainter.new(nil)
    m = QWMatrix.new 	  #// copy user specified transform
    textYPos = 0  #// distance from boundingRect y pos to baseline
    textXPos = 0  #// distance from boundingRect x pos to text start
    fm = fontMetrics()  	#// get widget font metrics
    if ( pixmapMode() )
	r = @pix.rect()
    else
	br = fm.boundingRect( @text ) 	#// rectangle covering text
	r  = br
	textYPos = -r.y()
	textXPos = -r.x()
	br.moveTopLeft( QPoint.new( -br.width()/2, -br.height()/2 ) )
    end

    r.moveTopLeft( QPoint.new(-r.width()/2, -r.height()/2) )
	  #// compute union of new and old rect
	  #// the resulting rectangle will cover what is already displayed
	  #// and have room for the new text/pixmap
    @eraseRect = @eraseRect.unite( @mtx.mapQRect(r) )
    @eraseRect.moveBy( -1, -1 )  #// add border for matrix round off
    @eraseRect.setSize( QSize.new( @eraseRect.width() + 2,@eraseRect.height() + 2 ) )
    pw = QMIN(@eraseRect.width(),width())
    ph = QMIN(@eraseRect.height(),height())
    pm = QPixmap.new(QSize.new( pw, ph) ) 		#// off-screen drawing pixmap
    pm.fill( backgroundColor() )

    p.begin( pm )
    m.translate( pw/2, ph/2 ) 	#// 0,0 is center
    m = @mtx.mul(m)
    p.setWorldMatrix( m )
    if ( pixmapMode() )
	p.drawPixmap( -@pix.width()/2, -@pix.height()/2, @pix )
    else
	p.setFont( font() ) 		#// use widget font
	p.drawText( r.left() + textXPos, r.top() + textYPos, @text )
    end
#if 0
#    p.setPenQColor( QColor("red") )
#    p.drawRectQRect( br )
#endif

    p.end()
    GC.start

    xpos = width / 2  - pw / 2
    ypos = height / 2 - ph / 2

    bitBlt( self, xpos, ypos,pm, 0, 0, -1, -1 )	#// copy pixmap to widget

    @eraseRect = @mtx.mapQRect( r )

end

end


class FontSelect < QDialog
YOff = 35

def initialize( parent, name)
    super ( parent, name, 0 )
    @newFont = RSignal.new("Object")
    @f = QFont.new( "Charter", 48, QFont::Bold )
    radios = [
	"Light (25)", "Normal (50)", "DemiBold (63)",
	"Bold (75)", "Black (87)"]
    @fontInternal = QWidget.new( self )
    @fontInternal.setFont( @f )
    @fontInternal.hide()

    @familyL    = QLabel.new("", self, "familyLabel" )
    @sizeL      = QLabel.new("", self, "sizeLabel" )
    @family     = QLineEdit.new(	 self, "family" )
    @sizeE      = QLineEdit.new(	 self, "pointSize" )
    @italic     = QCheckBox.new("",self, "italic" )
    @underline  = QCheckBox.new("", self, "underline" )
    @strikeOut  = QCheckBox.new("", self, "strikeOut" )
    @apply      = QPushButton.new("", self, "apply" )
    @sizeS      = QSlider.new(0,99,10,0, QSlider::Horizontal, self,
			      "pointSizeSlider" )

    @familyL.setGeometry( 10, YOff + 10, 100,20 )
    @familyL.setText( "Family :" )

    @sizeL.setGeometry( 10, YOff + 40, 100, 20 )
    @sizeL.setText( "Point size :" )

    @family.setGeometry( 110, YOff + 10, 100, 20 )
    @family.setText( "Charter" )

    @sizeE.setGeometry( 110, YOff + 40, 100, 20 )
    @sizeE.setText( "48" )

    @sizeS.setGeometry( 220, YOff + 40, 100, 20 )
    @sizeS.setRange( 1, 100 )
    @sizeS.setValue( 48 )
    @sizeS.setTracking( false )
    Qt::connect( @sizeS, QSIGNAL("valueChanged(int)"), self,"newSize" )
    Qt::connect( @sizeS, QSIGNAL("sliderMoved(int)"), self,"slidingSize" )

    @italic.setGeometry( 10, YOff + 70, 80, 20 )
    @italic.setText( "Italic" )
    Qt::connect( @italic, QSIGNAL("clicked()"), self, "newItalic" )

    @underline.setGeometry( 110, YOff + 70, 80, 20 )
    @underline.setText( "Underline" )
    Qt::connect( @underline, QSIGNAL("clicked()"), self, "newUnderline" )

    @strikeOut.setGeometry( 210, YOff + 70, 80, 20 )
    @strikeOut.setText( "StrikeOut" )
    Qt::connect( @strikeOut, QSIGNAL("clicked()"), self,"newStrikeOut" )

    @apply.setGeometry( 235, YOff + 10, 70, 20)
    @apply.setText( "APPLY" )
    @apply.setDefault( TRUE )
    Qt::connect( @apply, QSIGNAL("clicked()"), self,"doApply" )

    @weight = QButtonGroup.new( "Weight", self, "weightGroupBox" )
    @weight.setGeometry( 10, YOff + 100, 120, 120 )
    Qt::connect( @weight, QSIGNAL("clicked(int)"), self, "newWeight" )
    wname = ""
    rb = []
    for i in 0..4 #( i = 0   i < 5   i++ ) {
	wname = "radioButton #{i}"
	rb[i] = QRadioButton.new("", @weight, wname )
	rb[i].setGeometry( 10, 15+i*20 , 95, 20 )
	rb[i].setText( radios[i] )
    end
    rb[3].setChecked( TRUE )

    families = [
	"Charter", "Clean", "Courier", "Fixed", "Gothic", "Helvetica",
	"Lucida", "Lucidabright", "Lucidatypewriter", "Mincho",
	"New century schoolbook", "Symbol", "Terminal", "Times", "Utopia",
	0
    ]

    @pSizes	 = [
	"8", "10", "12", "14", "18", "24", "36", "48", "72", "96", 0
    ]

    @familyPopup = QPopupMenu.new
    tmp = families
    i = 0
    while( tmp[i] != 0 )
	@familyPopup.insertItem( tmp[i] )
        i = i+1
    end

    @pSize = QPopupMenu.new
    tmp = @pSizes
    i = 0
    while( tmp[i] != 0 )
	@pSize.insertItem( tmp[i] )
        i = i+1
    end

    menu = QMenuBar.new( self )
    menu.move( 0, 0 )
    menu.resize( 350, 30 )
    menu.insertItem( "Family", @familyPopup )
    menu.insertItem( "Point size", @pSize )

    Qt::connect( @familyPopup, QSIGNAL("activated(int)"),self,"familyActivated")
    Qt::connect( @pSize, QSIGNAL("activated(int)"), self, "pSizeActivated" )

    mLabelStr = [
	"Family:", "Point size:", "Weight:", "Italic:"
    ]

    mGroup = QButtonGroup.new("", self, "metricsGroupBox" )
    mGroup.setTitle( "Actual font" )
    mGroup.setGeometry(140, YOff + 100, 230, 100)
    @metrics = []
    @metrics0 =[]
    @metrics1 = []
    @metrics[0] = @metrics0
    @metrics[1] = @metrics1
    for i in 0..3 #( i = 0   i < 4   i++ ) {
	wname = "MetricsLabel[#{i}][0]"
	@metrics[0][i] = QLabel.new("", mGroup, wname)
	@metrics[0][i].setGeometry(10, 15 + 20*i, 70, 20)
	@metrics[0][i].setText( mLabelStr[i] )

	wname = "MetricsLabel[#{i}][1]"
	@metrics[1][i] = QLabel.new("", mGroup, wname)
	@metrics[1][i].setGeometry(90, 15 + 20*i, 135, 20)
    end
    updateMetrics()
    catchEvent
end
def newFont
   @newFont
end

def newStrikeOut()
    @f.setStrikeOut( @strikeOut.isChecked() )
    doFont()
end

def doFont()
    xyz = @f
    xyz.setPointSize( @f.pointSize()+1 )
    xyz.setPointSize( @f.pointSize() )
    @fontInternal.setFont( xyz )
    updateMetrics()
end

def newUnderline()
    @f.setUnderline( @underline.isChecked() )
    doFont()
end

def newItalic()
    @f.setItalic( @italic.isChecked() )
    doFont()
end

def newFamily()
    @f.setFamily( @family.text() )
    doFont()
end

def newWeight( id )
    case( id )
	when 0
	    @f.setWeight( QFont::Light )
	when 1
	    @f.setWeight( QFont::Normal )
	when 2
	    @f.setWeight( QFont::DemiBold )
	when 3
	    @f.setWeight( QFont::Bold )
	when 4
	    @f.setWeight( QFont::Black )
	else
	    return
    end
    doFont()
end

def newSize( value )
    tmp = "#{value}"
    @sizeE.setText( tmp )
    @f.setPointSize( value )
    doFont()
end

def slidingSize( value )
    tmp = "#{value}"
    @sizeE.setText( tmp )
end

def doApply()
    sz = @sizeE.text.to_i
    if ( sz > 100)
	@sizeS.blockSignals( TRUE )
	@sizeS.setValue( 100 )
	@sizeS.blockSignals( FALSE )
	@f.setPointSize( sz )
    else
	@sizeS.setValue( @sizeE.text.to_i )
    end
    @f.setFamily( @family.text() )
    doFont()

    @newFont.sendWith( @fontInternal.font() )
end

def familyActivated( id )
    @family.setText( @familyPopup.text(id) )
    newFamily()
end

def pSizeActivated( id )
    value = @pSize.text( id ).to_i
    @sizeS.blockSignals( TRUE )
    @sizeS.setValue( value )
    @sizeS.blockSignals( FALSE )
    newSize( value )
end

def updateMetrics()
    fi = @fontInternal.fontInfo()
    @metrics[1][0].setText( fi.family() )
    @metrics[1][1].setNum( fi.pointSize() )
    @metrics[1][2].setNum( fi.weight() )
    @metrics[1][3].setText( fi.italic().to_s )
end
end
=begin
/*
    Grand unifying widget, putting ShowXForm and XFormControl
    together.
*/
=end
class XFormCenter < QWidget

def initialize
    catchEvent
end

def setFont( f )
    @sx.setFont( f )
end

def resizeEvent( ev )
    @sx.resize( width() - 120, height() )
    @xc.resize( 120, height() )
end

def newMode( showPix )
    first = true

    if ( @sx.pixmapMode() == showPix )
	return
    end
    if ( showPix && first )
	first = false
	pm = QPixmap.new(nil)
	pm.load( "image.any" )
	@sx.setPixmap( pm )
	return
    end
    @sx.setPixmapMode( showPix )
end

def initialize( parent, name )
    @sx = ShowXForm.new(self,"")
    @sx.move( 120, 0 ) 		    #// the size is set by resizeEvent
    @xc = XFormControl.new(self,"")
    @xc.move( 0, 0 ) 		    #// the size is set by resizeEvent
    @xc.setFrameStyle( QFrame::Box | QFrame::Sunken )
    @xc.setLineWidth( 2 )
    Qt::connect( @xc.newText, @sx, "setText" )
    Qt::connect( @xc.newMatrix,@sx,"setMatrix" )
    Qt::connect( @xc.newFont, @sx,"setFont")
    Qt::connect( @xc.newMode, self,"newMode" )
    @sx.setText( "Troll" )
    catchEvent
end
end

   a = QApplication.new( [$0]+ARGV )

#if 0
=begin
    x = QColor.new( 0xaa, 0xbe, 0xff ) 	#// kind of blue
    g = QColorGroup.new( QColor("black"), x, x.light(), x.dark(), x.dark(120),	QColor("black"), QColor("white") )
    p = QPalette.new(QColor("white"))
    p.setNormal(g)
    p.setDisabled(g)
    p.setActive(g)
    QApplication::setPalette( p )
#endif
=end

    xfc = XFormCenter.new(nil,"")
    xfc.setGeometry( 0, 0, 500, 400 )

    a.setMainWidget( xfc )
    xfc.show()
    a.exec()


